/*
 * Uloha1Test.java
 * JUnit based test
 *
 */
package cz.muni.fi.pb138.jaro2013.uloha1;

import java.io.IOException;
import java.net.URI;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.dom.Document;

/**
 *
 * @author originally Petr Adamek 2008, 
 * changes by Tomas Pitner, Feb 2011, Feb 2013
 */
public class Uloha1Test extends TestCase {
    
    private static final String PEOPLEXML = "people.xml";
    
    private static final double EPSILON = 0.01;

    private Main reseniUlohy1;
    
    private XPath xpath;

    public Uloha1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        URI uri = Uloha1Test.class.getResource(PEOPLEXML).toURI();
        if (uri == null) {
            throw new IOException("The XML file for testing cannot be found.");
        }
        reseniUlohy1 = Main.newInstance(uri);

        xpath = XPathFactory.newInstance().newXPath();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(Uloha1Test.class);
        return suite;
    }

    /**
     * Test of changeName method, of class Main
     */
    public void testSellCarsToPolakAndNovak() throws XPathExpressionException {
        int novakCarsCount1 = getCarsCount("1");
        assertEquals("Novak should initially have 1 car", 1, novakCarsCount1);

        int polakCarsCount1 = getCarsCount("2");
        assertEquals("Polak should initially have 0 cars", 0, polakCarsCount1);

        int hrozaCarsCount1 = getCarsCount("3");
        assertEquals("Hroza should initially have 4 cars", 4, hrozaCarsCount1);

        reseniUlohy1.sellCarTo("6B6 82-51", "2");

        int polakCarsCount2 = getCarsCount("2");
        assertEquals("Polak should now have 1 car", 1, polakCarsCount2);
 
        int hrozaCarsCount2 = getCarsCount("3");
        assertEquals("Hroza should now have 3 cars", 3, hrozaCarsCount2);
        
        reseniUlohy1.sellCarTo("ZNH 28-51", "1");

        int novakCarsCount2 = getCarsCount("1");
        assertEquals("Novak should now have 2 cars", 2, novakCarsCount2);
 
        int hrozaCarsCount3 = getCarsCount("3");
        assertEquals("Hroza should now have 2 cars", 2, hrozaCarsCount3);
        
     }

    /**
     * Test of getWageCosts method, of class Main
     */
    public void testGetAverageCarPrice() throws XPathExpressionException {
        assertEquals(getAverageCarPrice(reseniUlohy1.getDocument()), reseniUlohy1.getAverageCarPrice(), EPSILON);
    }

    private double getAverageCarPrice(Document doc) throws XPathExpressionException {
        return ((Double) xpath.evaluate(
                "sum(//price)", reseniUlohy1.getDocument(), XPathConstants.NUMBER)) / ((Double) xpath.evaluate(
                "count(//price)", reseniUlohy1.getDocument(), XPathConstants.NUMBER));
    }

    private int getCarsCount(String personId) throws XPathExpressionException {
        double carsCount = (Double) xpath.evaluate("count(//person[@id='"+personId+"']/cars/car)",
                reseniUlohy1.getDocument(), XPathConstants.NUMBER);
        return (int) carsCount;
    }

}
